<?php
// Classes and libraries for module system
//
// webtrees: Web based Family History software
// Copyright (C) 2012 webtrees development team.
// Derived from PhpGedView
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// Module Display Sosa - Copyright (C) 2012 André DUMAS
// sosa-2-5-2 for WT 1-5-2  AD  17-03-2014
//
//  ← ← ← ← ← ← Note that I'm not a programmer! ← ← ← ← ←
// ↓ http://gustine.eu/wt/
// ↓ Diverses modifications pour webtrees 1.5.3 puis 1.6.0
// ↓ MR 2014-06 : remplacé WT_USER_IS_ADMIN par \WT\Auth::isAdmin() et WT_USER_CAN_EDIT par \WT\Auth::isEditor()
// ↓ MR 2014-10 : remplacé get_gedcom_setting(WT_GED_ID, et get_gedcom_setting($ged_id, par $tree->getPreference(
// ↓              ajouté appel aux fichiers de traduction en langue locale (FR.mo).
//  → → → → → → → → → → → → → → → → → → → → → → → → → → →
// Sosa 2-6-0 for WT 1.-6-0   AD October 2014
// with a button to change De Cujus 
// and update table ##sosa after adding ids
// keep with no change modifications introduced by MR (thanks (-_+)
//-------------------------------------------------------
// http://www.adumas.1s.fr/wt/   
// AD  July 2015
// Sosa 2-7-1 for WT 1.7.1  
// Many changes: add a class SosaClass including the functions for Sosa
//============================================
 /*
 *	Sosa V-2-7-2-01  for Webtrees 1.7.2
 *	 Small update to provide compatibility with 
 *		Module orphans which is using table ##sosa 
 *	André DUMAS FRANCE DEC 2015
 *	
 *	Sosa V-2-7-2-02  for Webtrees 1.7.4
 *	Small changes on DB update and popups
 *	André MAY 2016
 *
 *	Sosa V-2-7-4-1
 *   for WT 1.7.4-1
 *  add a navigation bar for sosa only
 *  Andre DUMAS 30-04-2016 FRANCE
 *
 *	Sosa V-2-7-9-1 for WR 1.7.9 +
 *		change code and add features
 *		André 25-05-2017 F
 ******************
 *	V-2-7-9-2   include version for visitor
 *******************
 *  V-2-7-9-3  update alert   JUL 2017
 *  =.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=
 *  =	André DUMAS FRANCE				  =
 *  = http://www.adumas.1s.fr/wt/		  =
 *  =  http://www.adumas.1s.fr/download/  =
 *  = contact: a.dumas1  at  free.fr	  =
 *  =.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=.=
 */

namespace Andre\WebtreesAddOn\Sosa;

use Composer\Autoload\ClassLoader;
use Fisharebest\Webtrees\Controller\BaseController;
use Fisharebest\Webtrees\Controller\PageController;
use Fisharebest\Webtrees\Filter;
use Fisharebest\Webtrees\I18N;
use Fisharebest\Webtrees\Log;
use Fisharebest\Webtrees\Module;
use Fisharebest\Webtrees\Module\AbstractModule;
use Fisharebest\Webtrees\Module\ModuleMenuInterface;
use Fisharebest\Webtrees\Module\ModuleSidebarInterface;
use Fisharebest\Webtrees\Module\ModuleConfigInterface;
use Fisharebest\Webtrees\Individual;
use Fisharebest\Webtrees\Menu;
use Fisharebest\Webtrees\Auth;
use Fisharebest\Webtrees\Tree;
use Fisharebest\Webtrees\File;
use Fisharebest\Webtrees\Date;
use Fisharebest\Webtrees\Database;
use Fisharebest\Webtrees\Functions;
use Fisharebest\Webtrees\Functions\FunctionsPrint;
use Fisharebest\Webtrees\Session;


use Andre\WebtreesAddOn\Sosa\myUpdateFunctions;

use Andre\WebtreesAddOn\Sosa\Template\AdminTemplate;
use Andre\WebtreesAddOn\Sosa\Template\PdfSosaTemplate;

class sosaModule extends AbstractModule implements ModuleConfigInterface, ModuleSidebarInterface {
	
	CONST CUSTOM_VERSION = '2-7-9-8b';
	CONST CUSTOM_VERSION_DATE = '10-04-2018';
	CONST CUSTOM_VERSION_AUTHOR = 'André DUMAS FRANCE';
	const CUSTOM_WEBSITE  = 'http://www.adumas.1s.fr/wt';
	
	CONST  SCHEMA_TARGET_VERSION = 4;
	
	var $css = 'sosa.css';
	public $ancestors = array();
	public $ancetres = array();
	
	public function __construct() {
		parent::__construct(__DIR__);

		// register the namespaces
		$loader = new ClassLoader();
		$loader->addPsr4('Andre\\WebtreesAddOn\\Sosa\\', WT_MODULES_DIR . $this->getName() . '/app');
		$loader->register();			
	}

	/**
	 * Get the module class.
	 * JustCarmen Thanks (-_+)
	 * Class functions are called with $this inside the source directory.
	 */
	public function module() {
		return new sosaClass;
	}
	private function template() {
		return new sosaSidebarTemplate;
	}
	
	private function a_template() {
		return new AdminTemplate;
	}
	
		private function sosa_listes() {
		return new sosaLists;
	}
	
		public function pdfout() {
		return new PdfSosaTemplate();
	}
	
	// Extend Module
	// = = = = = = = = = = = = = = = =
	public function getTitle(){
		$title = 'Sosa ';
		return $title; /* I18N: Name of a module/sidebar I18N::translate('Sosa:   '); */
	}
	
	// = = = = = = = = = = = = = = = =
	public function getDescription(){
		return I18N::translate('A sidebar showing an individual’s sosa.');
	}

	// = = = = = = = = = = = = = = = =
	public function defaultSidebarOrder(){
		return 1;
	}

	// Implement WT_Module_Config
	public function getConfigLink() {
		return 'module.php?mod=' . $this->getName() . '&amp;mod_action=admin_config';
	}
	// = = = = = = = = = = = = = = = =
	public function hasSidebarContent(){
		return true;
	}

	// = = = = = = = = = = = = = = = =
	public function getSidebarContent(){
		global $controller,  $WT_TREE, $show_full, $edit, $GEDCOM, $pid;
		
		// Update the database tables if neccessary.
		Database::updateSchema('\Andre\WebtreesAddOn\Sosa\Schema', 'SOSA_SCHEMA_VERSION', self::SCHEMA_TARGET_VERSION);	
			
		$pid = Filter::get('pid'); 
		$show_full = "1";
		$gidnum = $WT_TREE->getTreeId(); // indice du gedcom courant
				
		$user_id = sosaClass::getAuthId();
		if ($user_id == 0){
			$rid_sosa = sosaClass::readRootFromTable($WT_TREE->getTreeId());
			$rid = $rid_sosa;
		} else {
			if (($rid = $WT_TREE->getUserPreference(Auth::user(), 'gedcomid')) == null){$rid = 'a';}			
			if (($rid_sosa = sosaClass::readRootFromTable($WT_TREE->getTreeId())) == null){$rid_sosa = 'b';}
		}
		
		
		//for deskTab and update
		if (!$this->getSetting('CUSTOM_VERSION')  || ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION )  ){
			$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION);
		}
	
		// modification du titre
		$js = " 
			var tree = '".Filter::get('ged')."';
			var pid = '".Filter::get('pid')."';
			var root = '".$rid."';
			var roottable = '".$rid_sosa."';
			var userid = '".Auth::Id()."';
			var Father = '".I18N::translate('Father')."';
			var Mother = '".I18N::translate('Mother')."';
			var Child = '".I18N::translate('Child')."';
			var Husband = '".I18N::translate('Husband')."';
			var Wife = '".I18N::translate('Wife')."';
			var ChooseDeCujus = '".I18N::translate('Choose a de-cujus')."';
			var navDiv = ".$this->getSetting('SOSA_NAVIGATION_BAR').";
			var title = '".$this->module()->titre($pid, $gidnum)."';
			var sosaNum = title.split(' ')[2] ;
			if (sosaNum == 'none'){navDiv = false;}
		     $('#sosa').append(title);
			$('#sosa').wrapInner('<b></b>');			
		";
		/* navigation bar */
		$myjs = WT_MODULES_DIR.$this->getName().'/js/sosa.js';
				
		$controller->addInlineJavaScript($js)
					->addExternalJavascript($myjs)
					->addInlineJavascript('
							function include_css(css_file) {
								var html_doc = document.getElementsByTagName("head")[0];
								var css = document.createElement("link");
								css.setAttribute("rel", "stylesheet");
								css.setAttribute("type", "text/css");
								css.setAttribute("href", css_file);
								html_doc.appendChild(css);
							}
							var modName = \''.$this->getName().'\';							
							include_css("' . WT_MODULES_DIR . $this->getName() . '/' . 'css/' . $this->css.'");							
						')
		;
		
	
		$styleadd = "";
		$person = Individual::getInstance($rid, $WT_TREE);
		
		$details = 'In the numbering of Sosa-Stradonitz (see Ahnentafel in ' .
					'Wikipedia, number&nbsp;1 is assigned to the root of the family tree, called ' .
					'de-cujus';

		ob_start();			
	echo '<div id="mysosa">
		<div><table id="tablesosa">
			<tr>';
		
		if ( (!null == $person) && ($person->canShow() )){	
		echo 	'<td class="center facts_label' . $styleadd . ' "  colspan="2" title="' .
					 I18N::translate($details) .  '" 
					 alt="de-cujus (sosa 1):' . strip_tags($person->getFullName()) . '" >' .					
					'<a href="individual.php?pid=' . $person->getXref() . '&ged=' . $WT_TREE->getName() . 
					'"><b><p>de-cujus (sosa 1):   </p>' . strip_tags($person->getFullName()) . 
					'</b></a>
				</td>';
		} else {
		echo 	'<td class="center facts_label' . $styleadd . ' "  colspan="2" title="' .
					 I18N::translate('This id is the de-cujus: ') . '" 
					 alt="de-cujus (sosa 1):' . I18N::translate('private') . '" >' .					
					'<a href=" 
					 "><b><p>de-cujus (sosa 1):   </p>' . I18N::translate('private') . 
					'</b></a>
				</td>';	
		}		
		echo	'</tr>';
	if (( Auth::isAdmin() || Auth::isEditor($WT_TREE) || Auth::check() )) { 		
			
	echo	'<tr>
				<td class="center facts_label' . $styleadd . ' " style="width:50%;" title="'; 
					echo I18N::translate('Change de&nbsp;cujus');
	echo			'" alt="';
					echo I18N::translate('Change de&nbsp;cujus');
	echo 			'">
					<a class="famnav_link" href="#">
					<b><span id="spancdc">';
	echo		 	 I18N::translate('Change de&nbsp;cujus');
	echo 			'</span></b>
					</a>
				</td>
				<td class="center facts_label' . $styleadd . ' " style="width:50%;" title="'; 
					echo I18N::translate('Update sosa');
	echo			'" alt="';
					echo I18N::translate('Update sosa');
	echo 			'">
					<a class="famnav_link" href="#">
							<b><span id="spanupds">';
							 echo I18N::translate('Update sosa'); 
	echo					'</span></b>				</a>
				</td>
			</tr>
			<tr>
					<td class="center facts_label' . $styleadd . ' " title="'; 
					echo I18N::translate('Enter sosa number');
	echo			'" alt="';
					echo I18N::translate('Enter sosa number');
	echo 			'">
						<p><a class="famnav_link" href="#">
							<b><span id="spanins">';
							 echo I18N::translate('Enter sosa number'); 
	echo					'</span></b>
						</a></p>
						<input type="text" id="sosainput" value="" style="width:60%;background-color:white;"/>
						<input type="submit" id="inputsosaok" value="OK" />
					</td>
					<td class="center facts_label' . $styleadd . ' " title="'; 
					echo I18N::translate('This id as de-cujus');
	echo			'" alt="';
					echo I18N::translate('This id as de-cujus');
	echo 			'">
						<a class="famnav_link" href="#">
							<b><span id="spanthis">'; 
							 echo I18N::translate('This id as de-cujus'); 
	echo					'</span></b>
						</a>
					</td>
				</tr>
				<tr>
					<td class="center facts_label' . $styleadd . ' "  colspan="2" title="' .
					 I18N::translate('List of sosa html/txt/pdf ') . '" 
					 alt="' . I18N::translate('List of sosa html/txt/pdf') . '" >' .					
					'<a class="lsosa" href=" 
					 "><b><p>' . I18N::translate('List of sosa html/txt/pdf ') . '</p>' . 
					'</b></a>
				</td>
				</tr>
				';
	}			
	echo'</table>
			
	</div>';	
	echo '
		<div class="overlay" style="display:none;"></div>
			<div id="choosedecujus" class="popup_block">';
				if (Auth::check()){
					echo $this->module()->chooseNewDeCujus();
				}
	echo	'</div>';
	echo	'<div id="sosawait" class="popup_block">
				<p>';
				echo	I18N::translate('Wait, please...');
	echo		'</p>
				<img src="' . WT_MODULES_DIR . $this->getName() . '/images/progress2.gif" class="imgwait"/> 
			</div>
		</div>
	'; //my_sosa
	return	ob_get_clean();
		
} // fin getSidebarContent

	public function getSidebarAjaxContent(){
		return '';
	}

		public function modAction($mod_action){
		global $WT_TREE;
		switch ($mod_action) {
		case 'admin_config':  
			// Update the database tables if neccessary.
			Database::updateSchema('\Andre\WebtreesAddOn\Sosa\Schema', 'SOSA_SCHEMA_VERSION', self::SCHEMA_TARGET_VERSION);	
			//for deskTab and update
			if (!$this->getSetting('CUSTOM_VERSION')  || ( $this->getSetting('CUSTOM_VERSION') != self::CUSTOM_VERSION )  ){
				$this->setSetting('CUSTOM_VERSION', self::CUSTOM_VERSION);
			}
			echo $this->a_template()->pageContent();
			break;
			case 'check_version':
				$template = new AdminTemplate;
				$template->forceUpdate();
			break;
		case 'sosaDisplay':
			$sosa  = Filter::post('sosa');
			 echo $this->module()->getSosaXref($sosa);
			break;

		case 'sosaNav':
			$num = Filter::post('sosa');
			echo sosaClass::navSosa($num);
		break;
		
		case 'sosaUpdate':
			$pid = Filter::post('pid');
			if ($pid === null){$pid = Filter::get('pid');
			} elseif ( $pid == 'root'){
				$pid = sosaClass::readRootFromTable($WT_TREE->getTreeId());
			}		
			//user_id user pour lequel est créée la table sosa  admin|editor|member...
			if (Auth::check() ){
				$user_id = Auth::Id();
				$tree = $WT_TREE;
				$person = Individual::getInstance($pid , $tree);
				if (!null == $person){
					//$ancetres = self::getMyAncestors($person, 1);
					
					$this->module()->loadAncestors($person, 1);
					$ancetres =  sosaClass::$ancestors1; 
					ksort($ancetres);
					sosaClass::$ancestors1 = array();
										
					echo sosaClass::updateTableOfSosa($pid, $WT_TREE->getTreeId() , $user_id , $ancetres);
				}
			}	
		 
		break;
		
		case 'choosedecujus':
			if (Auth::check()){
					echo $this->module()->chooseNewDeCujus();
			} else {
				echo false;
			}
		break;	
		case 'lsosa':
			echo $this->sosa_listes()->displaySosaList();
			
		break;
		case 'exporttxt':
			$t = Filter::post('itab') + 4;			
			$this->sosa_listes()->getTextFile(Session::get('asc'), $t);
		break;
		
		case 'saveTab': 
		$h = Filter::post('sheader'); 
		$tab = Filter::post('tab'); 
			$tmp_dir = WT_DATA_DIR . 'tmp-' . $WT_TREE->getName() . '/';
			$tmp_file = 'sosa.html';
			$file = $tmp_dir . $tmp_file;
				$handle = fopen($file, 'w+');
				fwrite($handle, $h . $tab);
				fclose($handle);	
					echo  true;
		break;
		case 'exportpdf': 
			$this->pdfout()->pageBody('A4') ;			
		break;
		case 'download':
				$subdir 	= 'tmp-' . $WT_TREE->getName();
				$file 		= Filter::get('file'); 
				$this->downloadFile($file);
			break;
		default:
			echo '<br> Erreur dans la fonction modAction : mod_action = ' . $mod_action . '<br>'; break;
			//			header('HTTP/1.0 404 Not Found');
		}
	}

	protected function downloadFile($tmp_file){
		global $WT_TREE;
		$tmp_dir = WT_DATA_DIR . 'tmp-' . $WT_TREE->getName() . '/';
			//$tmp_file = 'sosa.txt';
			$file = $tmp_dir . $tmp_file;		// What to call the downloaded file
		if (file_exists($file)) {
			header('Content-Description: File Transfer');
			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="' . basename($file) . '"');
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: ' . filesize($file) );
			readfile($file);  
			File::delete($file);
			File::delete($tmp_dir . 'sosa.html');
			exit;
		}	

	}
	
	 
	// = = = = = = = = = = = = = = = = TabContent Orphans pour Sosa
	// Extend class WT_Module
	public function defaultAccessLevel() {
		return Auth::PRIV_PRIVATE;
		}
	///

	// Implement WT_Module_Sidebar
	public function isGrayedOut() {
		return false;
	}

	
		// = = = = = = = = = = = = = = = =
		// Actions from the configuration page		
	protected function confSosaVisitor(){
		global $WT_TREE;
		$ancetres = array();
		if (null == Filter::post('xref') ){return;}
		$tree = $WT_TREE;
		if (null !== (Filter::post('NEW_SOSA_TREE')) ){
			$pid = Filter::post('xref');				
			$tree = Tree::findByName(Filter::post('NEW_SOSA_TREE'));	
			$treeid = $tree->getTreeId();	
		} 	
		$user_id = 0; //visitor	
		$person = Individual::getInstance($pid , $tree);
		if (!null == $person){
			$this->module()->loadAncestors($person, 1);	
			$ancetres =  sosaClass::$ancestors1; 
					ksort($ancetres);
					sosaClass::$ancestors1 = array();
			sosaClass::updateTableOfSosa($pid, $treeid , $user_id , $ancetres);
			// old file?
			$this->delOldFileHtml($tree);
		}
		
	}
		

	private function delOldFileHtml($tree) {
		//tester Orphans installé???
		$module_name = 'orphans';
		if (!null == Module::getModuleByName($module_name)){
			//get conf param
			$subdir = Database::prepare("SELECT SQL_CACHE setting_value FROM `##module_setting` WHERE setting_name = ? AND module_name = ?")->execute(array('ORPHANS_CACHE_DIR',
												'orphans')
												)->fetchOne();
			$f  = Database::prepare("SELECT SQL_CACHE setting_value FROM `##module_setting` WHERE setting_name = ? AND module_name = ?")->execute(array('ORPHANS_CACHE_FILE',
											'orphans')
											)->fetchOne();
			
		} else {
			//not installed -> default param
			$subdir = 'orphans_cache';
			$f = 'orphans.html';
		}
		
		$file = explode('.',$f);
		$file_name = $file[0];
		$file_ext = $file[1];
		if (Auth::check()){	
			if (!is_dir($subdir)){
				File::mkdir($subdir);
				return  true;
			} 
	
		$filename = WT_DATA_DIR . $subdir . '/' . $file_name . '_0_' . $tree->getTreeId() . '.' . $file_ext; 	
			if (file_exists($filename)){
				File::delete($filename);
			}
		return true;
		} else {
			return false;
		}
	}
	
		
	// = = = = = = = = = = = = = = = =
		/*
	*	param $file   xxx.yyy
	*	param $subdir  dir from DATA_DIR
	*/
	
	
}	
return new SosaModule;
